def retention(events_table, retention_type, start_date, where_clause):
  return """
  WITH population AS
      (SELECT client_id AS unique_id, DATE_TRUNC('{1}', date) AS cohort_date, COUNT(*)
       FROM {0}
       WHERE 1 = 1
       {3}
       GROUP BY 1, 2),

  activity AS
      (SELECT DATE_TRUNC('{1}', date) AS activity_date, client_id AS unique_id, cohort_date
       FROM {0}
       JOIN population
       ON population.unique_id = client_id
       WHERE DATE_TRUNC('{1}', date) >= (CURRENT_DATE - INTERVAL '91 days')
       AND DATE_TRUNC('{1}', cohort_date) >= (CURRENT_DATE - INTERVAL '91 days')
       {3}),

  population_agg AS
      (SELECT DATE_TRUNC('{1}', date) AS cohort_date, COUNT(DISTINCT client_id) AS total
       FROM {0}
       WHERE 1 = 1
       {3}
       GROUP BY 1)

  SELECT * FROM
      (SELECT date, day as week_number, value, total, MAX(day) over (PARTITION BY date) AS max_week_num
      FROM
          (SELECT activity.cohort_date AS date,
             DATE_DIFF('{1}', activity.cohort_date, activity_date) AS day,
             total,
             COUNT(DISTINCT unique_id) AS value
          FROM activity
          JOIN population_agg
          ON activity.cohort_date = population_agg.cohort_date
          WHERE activity_date >= activity.cohort_date
          AND activity.cohort_date > '{2}'
          GROUP BY 1, 2, 3))
  WHERE week_number < max_week_num
  ORDER BY date, week_number""".format(events_table, retention_type, start_date, where_clause), []

def all_events_weekly(events_table, start_date, where_clause, event_column):
  return """
    WITH weekly_events AS
      (SELECT DATE_TRUNC('week', date) AS week, COUNT(*)
      FROM {0}
      WHERE DATE_TRUNC('week', date) >= '{1}'
      {2}
      GROUP BY 1),

  event_counts AS
      (SELECT week, {3}, count FROM
          (SELECT *, RANK() over (PARTITION BY week ORDER BY count) AS rank FROM
              (SELECT DATE_TRUNC('week', date) AS week, {3}, COUNT(*)
              FROM {0}
              WHERE DATE_TRUNC('week', date) >= '{1}'
              {2}
              GROUP BY 1, 2
              ORDER BY 1, 2))
      WHERE rank <= 20)

  SELECT weekly_events.week, event_counts.{3}, event_counts.count / weekly_events.count::FLOAT * 100 AS rate
  FROM weekly_events
  LEFT JOIN event_counts
  ON weekly_events.week = event_counts.week""".format(events_table, start_date, where_clause, event_column), ["week", "rate", event_column]

def active_users(events_table, start_date, where_clause=""):
  return """
    WITH weekly AS
    (SELECT day, COUNT(DISTINCT client_id) AS dist_clients
    FROM
      (SELECT DISTINCT date
       FROM {0}
       WHERE date >= '{1}'
       {2}
       ORDER BY date) AS g(day)
    LEFT JOIN {0}
    ON {0}.date BETWEEN g.day - 7 AND g.day
    AND {0}.date >= '{1}'
    {2}
    GROUP BY day
    ORDER BY day),

    monthly AS
    (SELECT day, count(DISTINCT client_id) AS dist_clients
    FROM
      (SELECT DISTINCT date
       FROM {0}
       WHERE date >= '{1}'
       {2}
       ORDER BY date) AS g(day)
    LEFT JOIN {0}
    ON {0}.date BETWEEN g.day - 28 AND g.day
    AND {0}.date >= '{1}'
    {2}
    GROUP BY day
    ORDER BY day),

    daily AS
    (SELECT date, COUNT(DISTINCT a.client_id) AS dau
     FROM {0} AS a WHERE date >= '{1}' {2} GROUP BY date),

    smoothed_daily AS
    (SELECT date as day,
           dau,
           AVG(dau) OVER(order by date ROWS BETWEEN 7 PRECEDING AND 0 FOLLOWING) as dist_clients
    FROM daily
    ORDER BY day desc)

    SELECT
      date,
      d.dist_clients as dau,
      w.dist_clients as wau,
      m.dist_clients as mau,
      (d.dist_clients::FLOAT / w.dist_clients) * 100.0 as weekly_engagement,
      (d.dist_clients::FLOAT / m.dist_clients) * 100.0 as monthly_engagement
    FROM {0} a
    JOIN smoothed_daily d on d.day = date
    JOIN weekly w on w.day = date
    JOIN monthly m on m.day = date
    WHERE date < current_date and date >= '2016-05-10'
    GROUP BY date, d.dist_clients, wau, mau
    ORDER BY date, dau, wau, mau""".format(events_table, start_date, where_clause), ["date", "dau", "wau", "mau", "weekly_engagement", "monthly_engagement"]



import logging

from redash_client.constants import VizWidth
from redash_client.dashboards.SummaryDashboard import SummaryDashboard


class ActivityStreamExperimentDashboard(SummaryDashboard):
  # These are either strings representing both the measurement name
  # event being measured or a key value pair: {<measurement_name>: <events>}
  DEFAULT_EVENTS = ["CLICK", "SEARCH", "BLOCK", "DELETE",
                    {
                        "event_name": "Positive Interactions",
                        "event_list": ["CLICK", "BOOKMARK_ADD", "SEARCH"]}]
  UT_EVENTS = [
      "scalar_parent_browser_engagement_unique_domains_count",
      "scalar_parent_browser_engagement_active_ticks",
      "scalar_parent_browser_engagement_tab_open_event_count",
      "scalar_parent_browser_engagement_max_concurrent_tab_count",
      "scalar_parent_browser_engagement_unfiltered_uri_count"]
  UT_HOURLY_EVENTS = [
      "scalar_parent_browser_engagement_unique_domains_count",
      "scalar_parent_browser_engagement_tab_open_event_count",
      "scalar_parent_browser_engagement_max_concurrent_tab_count",
      "scalar_parent_browser_engagement_unfiltered_uri_count"]
  UT_MAPPED_HOURLY_EVENTS = [
      "scalar_parent_browser_engagement_navigation_searchbar",
      "scalar_parent_browser_engagement_navigation_about_newtab"
      "scalar_parent_browser_engagement_navigation_about_home"]
  MAPPED_UT_EVENTS = [
      "scalar_parent_browser_engagement_navigation_searchbar",
      "scalar_parent_browser_engagement_navigation_about_newtab",
      "scalar_parent_browser_engagement_navigation_about_home"]

  DEFAULT_EVENTS_TABLE = "assa_events_daily"
  URL_FETCHER_DATA_SOURCE_ID = 28
  DISABLE_TITLE = "Disable Rate"
  RETENTION_DIFF_TITLE = "Daily Retention Difference (Experiment - Control)"

  def __init__(self, redash_client, project_name, dash_name, exp_id,
               start_date=None, end_date=None):
    DASH_TITLE = "{project}: {dash}".format(
        project=project_name, dash=dash_name)
    super(ActivityStreamExperimentDashboard, self).__init__(
        redash_client,
        DASH_TITLE,
        self.DEFAULT_EVENTS_TABLE,
        start_date, end_date)

    logging.basicConfig()
    self._logger = logging.getLogger()
    self._logger.setLevel(logging.INFO)
    self._experiment_id = exp_id

    self._params["experiment_id"] = self._experiment_id
    self._logger.info((
        "ActivityStreamExperimentDashboard: {name} "
        "Initialization Complete".format(name=dash_name)))

  def _get_title(self, template_name):
    title = template_name.title().split(": ")
    if len(title) > 1:
      title = title[1]
    return title

  def _get_event_title_description(self, template, event):
    if type(event) == str:
      event_name = event.capitalize()
      event_string = "('{}')".format(event)
    else:
      event_name = event["event_name"]
      events = []
      for event in event["event_list"]:
        events.append("'{}'".format(event))
      event_string = "(" + ", ".join(events) + ")"

    self._params["event"] = event
    self._params["event_string"] = event_string
    title = description = self._get_title(template["name"]).replace(
        "Event", event_name)

    if template["description"]:
      description = template["description"].lower().replace(
          "event", event_name).capitalize()

    event_data = {
        "title": title,
        "description": description
    }
    return event_data

  def _create_options(self):
    options = {
        "parameters": []
    }

    for param in self._params:
      param_obj = {
          "title": param,
          "name": param,
          "type": "text",
          "value": self._params[param],
          "global": False
      }
      options["parameters"].append(param_obj)

    return options

  def _apply_non_event_template(self, template, chart_data, values=None):
    title = description = self._get_title(template["name"])

    if template["description"]:
      description = template["description"]

    self._add_template_to_dashboard(
        template,
        chart_data,
        title,
        VizWidth.WIDE,
        description
    )

  def _apply_event_template(self, template, chart_data,
                            events_list, events_table, title=None):
    for event in events_list:
      event_data = self._get_event_title_description(template, event)

      self._add_template_to_dashboard(
          template,
          chart_data,
          event_data["title"],
          VizWidth.REGULAR,
          event_data["description"],
      )

  def _add_template_to_dashboard(self, template, chart_data, title,
                                 viz_width, description):
    # Remove graphs if they already exist.
    if title in chart_data:
      self._logger.info(("ActivityStreamExperimentDashboard: "
                         "{template} graph exists and is being removed"
                         .format(template=title)))

      query_id = chart_data[title]["query_id"]
      widget_id = chart_data[title]["widget_id"]
      self.remove_graph_from_dashboard(widget_id, query_id)

    self._logger.info(("ActivityStreamExperimentDashboard: "
                       "New {title} graph is being added"
                       .format(title=title)))
    self._add_forked_query_to_dashboard(
        title,
        template["id"],
        self._params,
        viz_width,
        template["options"],
        template["type"],
        description
    )

  def _apply_functions_to_templates(
      self, template_keyword, events_list, events_table,
      events_function, general_function=None, title=None
  ):
    if events_table is None:
      events_table = self._events_table
    self._params["events_table"] = events_table

    templates = self.redash.search_queries(template_keyword)
    chart_data = self.get_query_ids_and_names()

    for template in templates:
      if "event" in template["name"].lower():
        self._logger.info((
            "ActivityStreamExperimentDashboard: "
            "Processing template '{template_name}'"
            .format(template_name=template["name"])))
        events_function(
            template,
            chart_data,
            events_list,
            events_table,
            title)
      else:
        self._logger.info((
            "ActivityStreamExperimentDashboard: "
            "Processing template '{template_name}'"
            .format(template_name=template["name"])))
        general_function(template, chart_data)

  def add_graph_templates(self, template_keyword,
                          events_list=None, events_table=None):
    self._logger.info(
        "ActivityStreamExperimentDashboard: Adding templates.")

    if events_list is None:
      events_list = self.DEFAULT_EVENTS

    self._apply_functions_to_templates(
        template_keyword,
        events_list,
        events_table,
        self._apply_event_template,
        self._apply_non_event_template
    )



import json
import time
import logging
import requests
from slugify import slugify

# Taking into account different versions of Python
try:
  from urlparse import urljoin
  from urllib import urlencode
except ImportError:  # pragma: no cover
  from urllib.parse import urlencode
  from urllib.parse import urljoin

from redash_client.constants import VizType, VizWidth, ChartType, TimeInterval


class RedashClient(object):
  BASE_URL = "https://sql.telemetry.mozilla.org/api/"
  MAX_RETRY_COUNT = 5

  class RedashClientException(Exception):
    pass

  def __init__(self, api_key):
    self._api_key = api_key
    self._url_params = urlencode({"api_key": self._api_key})

    logging.basicConfig()
    self._logger = logging.getLogger()
    self._logger.setLevel(logging.INFO)

  def get_slug(self, name):
    return slugify(name)

  def make_visualization_options(self, chart_type=None, viz_type=None,
                                 column_mapping=None, series_options=None,
                                 time_interval=None, stacking=None):

    # See the API doc for more details about visualization options:
    # https://people-mozilla.org/~ashort/redash_docs/api.html
    if viz_type == VizType.COHORT:
      return {
          "timeInterval": time_interval,
      }

    # It's a chart viz type.
    options = {
        "bottomMargin": 50,
        "columnMapping": column_mapping,
        "globalSeriesType": chart_type,
        "legend": {"enabled": True},
        "series": {"stacking": "normal" if stacking else None},
        "seriesOptions": series_options if series_options else {},
        "sortX": True,
        "xAxis": {"type": "datetime", "labels": {"enabled": True}},
        "yAxis": [{"type": "linear"}, {"type": "linear", "opposite": True}],
    }

    return options

  def _make_request(self, request_function, url, args={}):
    if not request_function:
      request_function = requests.post

    try:
      if request_function != requests.post:
        response = request_function(url)
      else:
        response = request_function(url, args)
    except requests.RequestException as e:
      raise self.RedashClientException(
          ("Unable to communicate with redash: {error}").format(error=e), e)

    if response.status_code != 200:
      raise self.RedashClientException(
          ("Error status returned: {error_code} {error_message}").format(
              error_code=response.status_code,
              error_message=response.content,
          ), response.status_code)
    try:
      json_result = json.loads(response.content), response
    except ValueError as e:
      raise self.RedashClientException(
          ("Unable to parse JSON response: {error}").format(error=e))

    return json_result

  def _get_new_query_id(self, name, sql_query, data_source_id, description):
    url_path = "queries?{0}".format(self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    new_query_args = json.dumps({
        "name": name,
        "query": sql_query,
        "data_source_id": data_source_id,
        "description": description,
    })

    json_result, response = self._make_request(
        requests.post, query_url, new_query_args)

    query_id = json_result.get("id", None)
    return query_id

  def _get_visualization(self, query_id):
    url_path = "queries/{0}?{1}".format(str(query_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    query_json_data, response = self._make_request(requests.get, query_url)
    query_visualizations = query_json_data.get("visualizations", [])

    visualization_data = None
    if len(query_visualizations) >= 1:
      visualization_data = query_visualizations[0]

    return visualization_data

  def _refresh_graph(self, query_id):
    # Refresh our new query so it becomes available
    url_path = "queries/{0}/refresh?{1}".format(
        str(query_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)
    self._make_request(requests.post, query_url)

  def create_new_query(self, name, sql_query,
                       data_source_id, description=None):
    query_id = self._get_new_query_id(
        name, sql_query, data_source_id, description)

    # If we can't get a query ID, the query has no table. Exit now.
    if not query_id:
      return None, None

    visualization = self._get_visualization(query_id)

    table_id = None
    if visualization:
      table_id = visualization.get("id", None)

    self._refresh_graph(query_id)

    return query_id, table_id

  def get_query_results(self, sql_query, data_source_id):
    url_path = "query_results?{0}".format(self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    get_query_results_args = json.dumps({
        "query": sql_query,
        "data_source_id": data_source_id,
    })

    # If this query is still not uploaded, we'll get a job ID.
    # Let's retry in 1 second.
    for attempt in xrange(self.MAX_RETRY_COUNT):
      json_response, response = self._make_request(
          requests.post, query_url, get_query_results_args)
      if "job" not in json_response:
        break

      time.sleep(1)

    rows = json_response.get(
        "query_result", {}).get("data", {}).get("rows", [])
    return rows

  def make_new_visualization_request(self, query_id, viz_type, options, title):
    url_path = "visualizations?{0}".format(self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    new_visualization_args = json.dumps({
        "type": viz_type,
        "name": title,
        "options": options,
        "query_id": query_id,
    })

    json_result, response = self._make_request(
        requests.post, query_url, new_visualization_args)
    visualization_id = json_result.get("id", None)
    return visualization_id

  def create_new_visualization(self, query_id, viz_type=VizType.CHART,
                               title="Chart", chart_type=None,
                               column_mapping=None, series_options=None,
                               time_interval=None, stacking=False):

    # Note: column_mapping is a dict of which field names to use for the x and
    # y axis. (e.g. {"event":"x","count":"y","type":"series"})
    if viz_type == VizType.CHART and (
       chart_type not in ChartType.allowed_chart_types or
       column_mapping is None):

      raise ValueError(("chart_type and column_mapping "
                        "values required for a Chart visualization"))

    # Note: time_interval is one of "daily", "weekly", "monthly"
    if (viz_type == VizType.COHORT and
       time_interval not in TimeInterval.allowed_time_intervals):

      raise ValueError(("time_interval value required for "
                        "a Cohort visualization"))

    if viz_type != VizType.CHART and viz_type != VizType.COHORT:
      raise ValueError("VizType must be one of: VizType.CHART, VizType.COHORT")

    options = self.make_visualization_options(
        chart_type, viz_type, column_mapping,
        series_options, time_interval, stacking)

    visualization_id = self.make_new_visualization_request(
        query_id, viz_type, options, title)
    return visualization_id

  def create_new_dashboard(self, name):
    slug = self.get_slug(name)

    # Check if dashboard exists
    url_path = "dashboards/{0}?{1}".format(slug, self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    new_dashboard_args = json.dumps({"name": name})

    try:
      json_result, response = self._make_request(
          requests.get, query_url)
      self._logger.info((
          "RedashClient: Dashboard {name} exists and has "
          "been fetched").format(name=name))
    except self.RedashClientException as ex:
      server_error_code = ex.args[1]
      if server_error_code == 404:
        self._logger.info((
            "RedashClient: Dashboard {name} does not exist. "
            "Creating a new one.").format(name=name))
        url_path = "dashboards?{0}".format(self._url_params)
        query_url = urljoin(self.BASE_URL, url_path)

        json_result, response = self._make_request(
            requests.post, query_url, new_dashboard_args)

    dashboard_id = json_result.get("id", None)
    return dashboard_id

  def get_public_url(self, dash_id):
    url_path = "dashboards/{0}/share?{1}".format(
        str(dash_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    json_result, response = self._make_request(requests.post, query_url)
    public_url = json_result.get("public_url", None)
    return public_url

  def publish_dashboard(self, dash_id):
    url_path = "dashboards/{0}?{1}".format(str(dash_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    publish_dashboard_args = json.dumps({"is_draft": False})

    self._make_request(requests.post, query_url, publish_dashboard_args)

  def remove_visualization(self, viz_id):
    url_path = "widgets/{0}?{1}".format(str(viz_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)
    self._make_request(requests.delete, query_url)

  def delete_query(self, query_id):
    url_path = "queries/{0}?{1}".format(str(query_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)
    self._make_request(requests.delete, query_url)

  def add_visualization_to_dashboard(self, dash_id, viz_id, viz_width):
    if viz_width != VizWidth.REGULAR and viz_width != VizWidth.WIDE:
      raise ValueError(("viz_width should be one of "
                        "VizWidth.WIDE or VizWidth.REGULAR"))

    url_path = "widgets?{0}".format(self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    add_visualization_args = json.dumps({
        "dashboard_id": dash_id,
        "visualization_id": viz_id,
        "width": viz_width,
        "options": {},
        "text": "",
    })

    self._make_request(requests.post, query_url, add_visualization_args)

  def update_query_schedule(self, query_id, schedule):
    url_path = "queries/{0}?{1}".format(str(query_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    update_query_args = json.dumps({"schedule": schedule, "id": query_id})

    self._make_request(requests.post, query_url, update_query_args)

  def update_query(self, query_id, name, sql_query,
                   data_source_id, description, options=None):
    url_path = "queries/{0}?{1}".format(str(query_id), self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    update_query_args = {
        "data_source_id": data_source_id,
        "query": sql_query,
        "name": name,
        "description": description,
        "id": query_id,
    }

    if options:
      update_query_args["options"] = options

    self._make_request(requests.post, query_url, json.dumps(update_query_args))
    self._refresh_graph(query_id)

  def fork_query(self, query_id):
    url_path = "queries/{0}/fork?{1}".format(query_id, self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    json_result, response = self._make_request(
        requests.post, query_url)
    fork = {
        "id": json_result.get("id", None),
        "query": json_result.get("query", None),
        "data_source_id": json_result.get("data_source_id", None)
    }

    return fork

  def search_queries(self, keyword):
    url_path = "queries/search?q={0}&{1}".format(keyword, self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    json_result, response = self._make_request(
        requests.get, query_url)
    print json_result

    templated_queries = []
    for query in json_result:
      query_id = query.get("id", None)
      visualization = self._get_visualization(query_id)
      options = visualization.get("options", None)
      viz_type = visualization.get("type", None)

      templated_queries.append({
          "id": query_id,
          "description": query.get("description", None),
          "name": query.get("name", None),
          "data_source_id": query.get("data_source_id", None),
          "options": options,
          "type": viz_type,
          "query": query.get("query", None)
      })

    return templated_queries

  def get_widget_from_dash(self, name):
    slug = self.get_slug(name)
    url_path = "dashboards/{0}?{1}".format(slug, self._url_params)
    query_url = urljoin(self.BASE_URL, url_path)

    # Note: row_arr is in the form:
    # [[{}, {}], [{}], ...]
    #
    # Where each sub-array represents a row of widgets in a redash dashboard

    get_widget_args = json.dumps({"name": name})

    json_result, response = self._make_request(
        requests.get, query_url, get_widget_args)
    row_arr = json_result.get("widgets", [])

    # Return a flattened list of all widgets
    widgets = []
    for row in row_arr:
      if len(row) == 1:
        widgets.append(row[0])
      elif len(row) == 2:
        widgets += [row[0], row[1]]

    return widgets



import json
import boto3
import urllib
from boto3.s3.transfer import S3Transfer

from datetime import datetime, timedelta

BUCKET = "telemetry-public-analysis-2"
client = boto3.client('s3', 'us-west-2')
transfer = S3Transfer(client)
s3 = boto3.client("s3")


def upload_as_json(directory_name, filename, data):
  path = "activity-stream/" + directory_name + "/"
  s3_key = path + filename

  json_data = json.dumps(data)
  file_path = "/tmp/{filename}".format(filename=filename)
  with open(file_path, 'w') as f:
    f.write(json_data)

  transfer.upload_file(
      file_path,
      BUCKET,
      s3_key,
      extra_args={"ContentType": "application/json"})

  return "https://analysis-output.telemetry.mozilla.org/" + s3_key


def read_experiment_definition(url):
  try:
    response = urllib.urlopen(url)
    return json.loads(response.read())
  except:
    return {}


def read_experiment_definition_s3(filename):
  DIRECTORY_NAME = "experiments/json_definitions"

  path = "activity-stream/" + DIRECTORY_NAME + "/"
  s3_key = path + filename

  obj = s3.get_object(Bucket="telemetry-public-analysis-2", Key=s3_key)

  try:
    experiments_string = obj["Body"].read()
    return json.loads(experiments_string)
  except:
    return {}


def format_date(date):
  date_epoch = datetime.fromtimestamp(date / 1000.0)
  date = date_epoch.strftime("%m/%d/%y")
  return date


# A date is considered "old" if it's > 3 days earlier than today.
def is_old_date(date):
  three_old = datetime.today() - timedelta(days=3)
  input_date_epoch = datetime.fromtimestamp(date / 1000.0)
  return input_date_epoch < three_old



import math
import statistics
from scipy import stats
import statsmodels.stats.power as smp

from redash_client.utils import upload_as_json
from redash_client.constants import TTableSchema, VizWidth
from redash_client.dashboards.ActivityStreamExperimentDashboard import (
    ActivityStreamExperimentDashboard)


class StatisticalDashboard(ActivityStreamExperimentDashboard):
  TTABLE_DESCRIPTION = (
    "Smaller p-values (e.g. <= 0.05) indicate a high "
    "probability that the variants have different distributions. Alpha "
    "error indicates the probability a difference is observed when one "
    "does not exists. Larger power (e.g. >= 0.7) indicates a high "
    "probability that an observed difference is correct. Beta error "
    "(1 - power) indicates the probability that no difference is observed "
    "when indeed one exists.")
  ALPHA_ERROR = 0.005
  TTABLE_TEMPLATE = {"columns": TTableSchema, "rows": []}


  def __init__(self, redash_client, project_name, dash_name, exp_id,
               start_date=None, end_date=None):
    super(StatisticalDashboard, self).__init__(
        redash_client,
        project_name,
        dash_name,
        exp_id,
        start_date,
        end_date)

    self._ttables = {}

  def _copy_ttable_tempalte(self):
    template_copy = self.TTABLE_TEMPLATE.copy()
    template_copy["rows"] = []
    return template_copy

  def _compute_pooled_stddev(self, control_std, exp_std,
                             control_vals, exp_vals):

    control_len_sub_1 = len(control_vals) - 1
    exp_len_sub_1 = len(exp_vals) - 1

    pooled_stddev_num = (pow(control_std, 2) * control_len_sub_1 +
                         pow(exp_std, 2) * exp_len_sub_1)
    pooled_stddev_denom = control_len_sub_1 + exp_len_sub_1

    pooled_stddev = math.sqrt(pooled_stddev_num / float(pooled_stddev_denom))
    return pooled_stddev

  def _power_and_ttest(self, control_vals, exp_vals):
    control_mean = statistics.mean(control_vals)
    control_std = statistics.stdev(control_vals)
    exp_mean = statistics.mean(exp_vals)
    exp_std = statistics.stdev(exp_vals)

    print "control mean & stddev"
    print control_mean
    print control_std
    print "exp mean & stddev"
    print exp_mean
    print exp_std

    pooled_stddev = self._compute_pooled_stddev(
        control_std, exp_std, control_vals, exp_vals)

    power = 0
    percent_diff = None
    if control_mean != 0 and pooled_stddev != 0:
      percent_diff = (control_mean - exp_mean) / float(control_mean)
      effect_size = (abs(percent_diff) * float(control_mean)) / float(pooled_stddev)
      power = smp.TTestIndPower().solve_power(
          effect_size,
          nobs1=len(control_vals),
          ratio=len(exp_vals) / float(len(control_vals)),
          alpha=self.ALPHA_ERROR, alternative='two-sided')

    ttest_result = stats.ttest_ind(control_vals, exp_vals, equal_var=False)
    p_val = ""
    if len(ttest_result) >= 2 and not math.isnan(ttest_result[1]):
      p_val = ttest_result[1]

    mean_diff = exp_mean - control_mean

    if p_val <= self.ALPHA_ERROR and mean_diff < 0:
      significance = "Negative"
    elif p_val <= self.ALPHA_ERROR and mean_diff > 0:
      significance = "Positive"
    else:
      significance = "Neutral"

    return {
        "power": power,
        "p_val": p_val,
        "control_mean": control_mean,
        "mean_diff": mean_diff,
        "percent_diff": 0 if percent_diff is None else percent_diff * -100,
        "significance": significance,
    }

  def _get_ttable_data_for_query(self, label, query_string,
                                 column_name, data_source_id):
    data = self.redash.get_query_results(
        query_string, data_source_id)

    if data is None or len(data) <= 3 or (column_name not in data[0]):
      return {}

    control_vals = []
    exp_vals = []
    for row in data:
      if "type" in row and row["type"].find("control") == -1:
        exp_vals.append(row[column_name])
      elif "type" in row:
        control_vals.append(row[column_name])
      else:
        return {}
    print len(control_vals)
    print len(exp_vals)

    results = self._power_and_ttest(control_vals, exp_vals)
    return {
        "Metric": label,
        "Alpha Error": self.ALPHA_ERROR,
        "Power": results["power"],
        "Two-Tailed P-value (ttest)": results["p_val"],
        "Control Mean": results["control_mean"],
        "Experiment Mean - Control Mean": results["mean_diff"],
        "Percent Difference in Means": results["percent_diff"],
        "Significance": results["significance"]
    }

  def _apply_ttable_event_template(self, template, chart_data, events_list,
                                   events_table, title):

    if title not in self._ttables:
      self._ttables[title] = self._copy_ttable_tempalte()

    self._params["events_table"] = events_table
    for event in events_list:
      event_data = self._get_event_title_description(template, event)
      options = self._create_options()

      adjusted_string = template["query"].replace(
          "{{{", "{").replace("}}}", "}")
      query_string = adjusted_string.format(**self._params)
      print query_string

      self.redash.update_query(
          template["id"],
          template["name"],
          template["query"],
          template["data_source_id"],
          event_data["description"],
          options
      )
      ttable_row = self._get_ttable_data_for_query(
          event_data["title"],
          query_string,
          "count",
          template["data_source_id"])

      if len(ttable_row) == 0:
        self._logger.info((
            "StatisticalTester: "
            "Query '{name}' has no relevant data and will not be "
            "included in T-Table.".format(name=event_data["title"])))
        continue

      self._ttables[title]["rows"].append(ttable_row)

    print self._ttables[title]

  def add_ttable_data(self, template_keyword, title,
                      events_list=None, events_table=None):
    self._logger.info((
        "StatisticalTester: Adding data for "
        "{keyword}").format(keyword=template_keyword))

    if events_list is None:
      events_list = self.DEFAULT_EVENTS
      events_table = self._events_table

    # Create the t-table
    self._apply_functions_to_templates(
        template_keyword,
        events_list,
        events_table,
        self._apply_ttable_event_template,
        None,
        title)

  def add_ttable(self, title):
    self._logger.info((
        "StatisticalTester: Creating a T-Table with "
        "title {title}").format(title=title))

    FILENAME = '{exp_id}_{title}'.format(exp_id=self._experiment_id, title=title)

    chart_data = self.get_query_ids_and_names()

    # Remove a table if it already exists
    if title in chart_data:
      self._logger.info((
          "StatisticalTester: "
          "Stale T-Table exists and will be removed"))
      query_id = chart_data[title]["query_id"]
      widget_id = chart_data[title]["widget_id"]
      self.remove_graph_from_dashboard(widget_id, query_id)

    query_string = upload_as_json("experiments", FILENAME, self._ttables[title])
    query_id, table_id = self.redash.create_new_query(
        title,
        query_string,
        self.URL_FETCHER_DATA_SOURCE_ID,
        self.TTABLE_DESCRIPTION,
    )
    self.redash.add_visualization_to_dashboard(
        self._dash_id, table_id, VizWidth.WIDE)



class RetentionType:
  DAILY = "day"
  WEEKLY = "week"


class TimeInterval:
  DAILY = "daily"
  WEEKLY = "weekly"
  MONTHLY = "monthly"
  allowed_time_intervals = [DAILY, WEEKLY, MONTHLY]


class VizType:
  CHART = "CHART"
  COHORT = "COHORT"


class VizWidth:
  REGULAR = 1
  WIDE = 2


class ChartType:
  BAR = "column"
  PIE = "pie"
  LINE = "line"
  SCATTER = "scatter"
  AREA = "area"
  allowed_chart_types = [BAR, PIE, LINE, SCATTER, AREA]


TTableSchema = [
    {
        "name": "Metric",
        "type": "string",
        "friendly_name": "Metric"
    }, {
        "name": "Alpha Error",
        "type": "float",
        "friendly_name": "Alpha Error"
    }, {
        "name": "Power",
        "type": "float",
        "friendly_name": "Power"
    }, {
        "name": "Two-Tailed P-value (ttest)",
        "type": "float",
        "friendly_name": "Two-Tailed P-value (ttest)"
    }, {
        "name": "Control Mean",
        "type": "float",
        "friendly_name": "Control Mean"
    }, {
        "name": "Experiment Mean - Control Mean",
        "type": "float",
        "friendly_name": "Experiment Mean - Control Mean"
    }, {
        "name": "Percent Difference in Means",
        "type": "float",
        "friendly_name": "Percent Difference in Means"
    }, {
        "name": "Significance",
        "type": "string",
        "friendly_name": "Significance"
    }]



import time

from redash_client.templates import retention, all_events_weekly, active_users
from redash_client.constants import (
    VizWidth, VizType, RetentionType, ChartType, TimeInterval)


class SummaryDashboard(object):
  TILES_DATA_SOURCE_ID = 5
  DAILY_RETENTION_TITLE = "Daily Retention"
  WEEKLY_RETENTION_TITLE = "Weekly Retention"
  EVENTS_WEEKLY_TITLE = "Weely Events"
  MAU_DAU_TITLE = "Engagement"
  MAU_DAU_SERIES_OPTIONS = {
      "mau": {
          "type": ChartType.AREA,
          "yAxis": 0,
          "zIndex": 0,
          "index": 0
      },
      "wau": {
          "type": ChartType.AREA,
          "yAxis": 0,
          "zIndex": 1,
          "index": 0
      },
      "dau": {
          "type": ChartType.AREA,
          "yAxis": 0,
          "zIndex": 2,
          "index": 0
      },
  }

  class SummaryDashboardException(Exception):
    pass

  def __init__(self, redash_client, dash_name, events_table_name,
               start_date, end_date=None):
    self._dash_name = dash_name
    self._events_table = events_table_name
    self._start_date = start_date
    self._end_date = end_date if end_date else time.strftime("%Y-%m-%d")
    self._params = {
        "start_date": self._start_date,
        "end_date": self._end_date
    }

    self.redash = redash_client
    self._dash_id = self.redash.create_new_dashboard(self._dash_name)
    self.redash.publish_dashboard(self._dash_id)
    self.public_url = self.redash.get_public_url(self._dash_id)

  def update_refresh_schedule(self, seconds_to_refresh):
    widgets = self.redash.get_widget_from_dash(self._dash_name)

    for widget in widgets:
      widget_id = widget.get(
          "visualization", {}).get("query", {}).get("id", None)

      if not widget_id:
        continue

      self.redash.update_query_schedule(widget_id, seconds_to_refresh)

  def get_query_ids_and_names(self):
    widgets = self.redash.get_widget_from_dash(self._dash_name)

    data = {}
    for widget in widgets:
      widget_id = widget.get("id", None)

      query_id = widget.get(
          "visualization", {}).get("query", {}).get("id", None)

      widget_name = widget.get(
          "visualization", {}).get("query", {}).get("name", None)

      widget_query = widget.get(
          "visualization", {}).get("query", {}).get("query", None)

      if not widget_name:
        continue

      data[widget_name] = {
          "query_id": query_id,
          "widget_id": widget_id,
          "query": widget_query,
      }

    return data

  def remove_graph_from_dashboard(self, widget_id, query_id):
    if widget_id is not None:
      self.redash.remove_visualization(widget_id)

    if query_id is not None:
      self.redash.delete_query(query_id)

  def remove_all_graphs(self):
    widgets = self.get_query_ids_and_names()

    for widget_name in widgets:
      widget = widgets[widget_name]
      widget_id = widget.get("widget_id", None)
      query_id = widget.get("query_id", None)

      self.remove_graph_from_dashboard(widget_id, query_id)

  def _get_mau_dau_column_mappings(self, query_fields):
    mau_dau_column_mapping = {
        # Date
        query_fields[0]: "x",
        # DAU
        query_fields[1]: "y",
        # WAU
        query_fields[2]: "y",
        # MAU
        query_fields[3]: "y",
    }
    engagement_ratio_column_mapping = {
        # Date
        query_fields[0]: "x",
        # Weekly Engagement
        query_fields[4]: "y",
        # Montly Engagement
        query_fields[5]: "y",
    }
    return mau_dau_column_mapping, engagement_ratio_column_mapping

  def _add_forked_query_to_dashboard(
      self, query_title, parent_query_id, query_params, visualization_width,
      options, visualization_type=VizType.CHART, visualization_name="Chart"
  ):

    fork = self.redash.fork_query(parent_query_id)
    adjusted_string = fork["query"].replace("{{{", "{").replace("}}}", "}")
    sql_query = adjusted_string.format(**query_params)

    self.redash.update_query(
        fork["id"],
        query_title,
        sql_query,
        fork["data_source_id"],
        "",
    )

    viz_id = self.redash.make_new_visualization_request(
        fork["id"],
        visualization_type,
        options,
        visualization_name,
    )
    self.redash.add_visualization_to_dashboard(
        self._dash_id, viz_id, visualization_width)

  def _add_query_to_dashboard(self, query_title, query_string,
                              data_source, visualization_width,
                              visualization_type=VizType.CHART,
                              visualization_name="", chart_type=None,
                              column_mapping=None, series_options=None,
                              time_interval=None, stacking=True):

    query_id, table_id = self.redash.create_new_query(
        query_title, query_string, data_source)
    viz_id = self.redash.create_new_visualization(
        query_id,
        visualization_type,
        visualization_name,
        chart_type,
        column_mapping,
        series_options,
        time_interval,
        stacking,
    )
    self.redash.add_visualization_to_dashboard(
        self._dash_id, viz_id, visualization_width)

  def add_mau_dau(self, where_clause=""):
    if self.MAU_DAU_TITLE in self.get_query_ids_and_names():
      return

    query_string, fields = active_users(
        self._events_table, self._start_date, where_clause)

    mau_dau_mapping, er_mapping = self._get_mau_dau_column_mappings(fields)

    # Make the MAU/WAU/DAU graph
    self._add_query_to_dashboard(
        self.MAU_DAU_TITLE,
        query_string,
        self.TILES_DATA_SOURCE_ID,
        VizWidth.WIDE,
        VizType.CHART,
        "",
        ChartType.AREA,
        mau_dau_mapping,
        series_options=self.MAU_DAU_SERIES_OPTIONS,
    )

    # Make the engagement ratio graph
    self._add_query_to_dashboard(
        self.MAU_DAU_TITLE,
        query_string,
        self.TILES_DATA_SOURCE_ID,
        VizWidth.WIDE,
        VizType.CHART,
        "",
        ChartType.LINE,
        er_mapping,
    )

  def add_retention_graph(self, retention_type, where_clause=""):
    time_interval = TimeInterval.WEEKLY
    graph_title = self.WEEKLY_RETENTION_TITLE

    if retention_type == RetentionType.DAILY:
      time_interval = TimeInterval.DAILY
      graph_title = self.DAILY_RETENTION_TITLE

    current_charts = self.get_query_ids_and_names()
    if graph_title in current_charts:
      return

    query_string, fields = retention(
        self._events_table, retention_type, self._start_date, where_clause)

    self._add_query_to_dashboard(
        graph_title,
        query_string,
        self.TILES_DATA_SOURCE_ID,
        VizWidth.WIDE,
        VizType.COHORT,
        time_interval=time_interval,
    )

  def add_events_weekly(self, where_clause="", event_column="event_type"):
    if self.EVENTS_WEEKLY_TITLE in self.get_query_ids_and_names():
      return

    query_string, fields = all_events_weekly(
        self._events_table, self._start_date, where_clause, event_column)

    column_mapping = {
        fields[0]: "x",
        fields[1]: "y",
        fields[2]: "series",
    }

    self._add_query_to_dashboard(
        self.EVENTS_WEEKLY_TITLE,
        query_string,
        self.TILES_DATA_SOURCE_ID,
        VizWidth.WIDE,
        VizType.CHART,
        "",
        ChartType.BAR,
        column_mapping,
        stacking=True
    )



import csv
from redash_client.utils import upload_as_json
from redash_client.constants import TTableSchema

schema = [
    {
        "name": "Date",
        "type": "string",
        "friendly_name": "Date"
    }, {
        "name": "Type",
        "type": "string",
        "friendly_name": "Type"
    }, {
        "name": "Average",
        "type": "float",
        "friendly_name": "Average"
    }, {
        "name": "Error",
        "type": "float",
        "friendly_name": "Error"
    }
]

FILENAME = 'active_ticks_existing_users'
TABLE_TEMPLATE = {"columns": schema, "rows": []}

with open('active_ticks_graph.csv') as ticks_file:
  meep = csv.reader(ticks_file, delimiter=',')
  print meep
  break




#upload_as_json("experiments", FILENAME, self._ttables[title])


from setuptools import setup
setup(
  name = 'redash_client',
  packages = ['redash_client', 'redash_client.dashboards'],
  version = '0.1.12',
  description = 'A client for the re:dash API for stmo (https://sql.telemetry.mozilla.org)',
  author = 'Marina Samuel',
  author_email = 'msamuel@mozilla.com',
  url = 'https://github.com/mozilla/redash_client',
  keywords = ['redash', 'experiments', 'a/b tests'],
  classifiers = [],
  install_requires=[
    "boto3 == 1.4.4",
    "requests == 2.12.1",
    "python-slugify == 1.2.4",
    "scipy == 1.0.0",
    "statistics == 1.0.3.5",
    "statsmodels == 0.6.1",
    "urllib3 == 1.21.1"
  ]
)